 /*
 * FILE --- veh_sim_lqr.cc
 *
 * PURPOSE --- To rpovide an example use of the class for 4'th
 * order Runge-Kutta simulation, RK4_SIM.
 * Implements the functions of a class for performing a simulation of
 * a simple vehicle with point mass dynamics.
 *
 */

// Include Files
#include "veh_sim_lqr.hh"

// Useful macros
#define SIGN(A)  ((A) >= (0) ? (1.0) : (-1.0))
#define SATURATE(u,MAX_VALUE) \
((u) > MAX_VALUE ? MAX_VALUE : ((u) < -MAX_VALUE ? -MAX_VALUE : (u)));

VEH_SIM_LQR::VEH_SIM_LQR (double _sampling_period, int _simulation_per_cycle,
			  double _initial_time, double *_initial_state) 
  : RK4_SIM(3, 3, (_sampling_period/((double)_simulation_per_cycle)), 
	    _initial_time, _initial_state)
{
  // samp_period >= integration_step
  
  // Initialize parameters
  sampling_period = _sampling_period;
  simulation_per_cycle = _simulation_per_cycle;
  mass = 1300;
  drag_x = 0.3;
  //  drag_y = 0.1;
  tau = 0.2;
  disturbance = 100.0;
  
  lqr_ctrl = new LQR_CTRL(3);
  
}

VEH_SIM_LQR::~VEH_SIM_LQR()
{
  delete lqr_ctrl;
}

inline void VEH_SIM_LQR::DERIV(const double t, const double *x, 
			       const double *u, double *xdot)
{
  // This is where the derivative is defined: xdot = f(t,x,u)
  // Simple point mass dynamics
  // delta_dot = V_veh - V_preceading;
  // V_veh_dot = (1.0/m)*(-Ap*(V_veh)^2 - d + f);
  // f_dot = (1.0/tau)*(-f + u);
  
  // x[0] = longitudinal position
  // x[1] = longitudinal velosity
  // x[2] = force in longitudinal direction
  // u[0] = reference velocity
  // u[1] = applied control input
  
  // Vehicle Dynamics
  xdot[0] = x[1] - u[0];
  xdot[1] = (1.0/mass)*(-drag_x*x[1]*x[1] - disturbance + x[2]);
  xdot[2] = (1.0/tau)*(-x[2] + u[1]);
  
  //  xdot[0] = (1.0/mass)*(-drag_x*(x[0] + u[2]) + u[0]*cos(u[1]));
  //  xdot[1] = x[2];
  //  xdot[2] = (1.0/mass)*(-drag_y*x[2] + u[0]*sin(u[1]));
}

void VEH_SIM_LQR::simulate()
{
  static double ref_vel = 45.0;
  static double lambda = 0.9;
  double err;
  static int count = 1000;
  
  err = - (plant_state[0] + lambda*plant_state[1]);

  ctrl_input[0] = ref_vel;
  ctrl_input[1] = lqr_ctrl->control(plant_state);
  
  for (int ii=0; ii < simulation_per_cycle; ii++)
    {
      solve();
    }
  
  // I use this just for saving the error
  ctrl_input[2] = err;
  
  if(count < 2000)
    ref_vel += 0.005;
  else if((count >= 3000) && (count < 5000))
    ref_vel -= 0.005;
  count++;
  if(count == 6000)
    count = 0;
  
}


